"""Implement gradient descent in 2d with momentum."""


import numpy as np

from optimize_2d import grad_paraboloid, paraboloid
from util import write_movie

if __name__ == "__main__":
    import matplotlib.pyplot as plt

    nx, ny = (301, 301)
    x = np.linspace(-3, 3, nx)
    y = np.linspace(-3, 3, ny)
    mx, my = np.meshgrid(x, y)
    pos = np.stack((mx, my))
    mz = paraboloid(pos)

    plt.contourf(mx, my, mz)
    plt.colorbar()

    start_pos = np.array((2.9, -2.9))
    step_size = 0.05
    alpha = 0.8
    step_total = 100

    pos_list = [start_pos]
    grad = np.array((0.0, 0.0))
    for _ in range(step_total):
        grad = grad * alpha - step_size * grad_paraboloid(pos_list[-1])
        pos = pos_list[-1] + grad
        pos_list.append(pos)

    for pos in pos_list:
        plt.plot(pos[0], pos[1], ".r")
    plt.show()

    write_movie(mx, my, mz, pos_list, "writer_grad_parabola_plot_momentum")
